@@ -6,8 +6,8 @@ SRCDIR=/usr/src/turbo/SOURCES
 USERID=`id -u`
 GRPID=`id -g`
 
-( cd ../../../.. ; chown -R ${USERID}.${GRPID} ${SRCDIR}/samba-2.2.4 )
-( cd ../../../.. ; tar czvf ${SRCDIR}/samba-2.2.4.tar.gz samba-2.2.4 )
+( cd ../../../.. ; chown -R ${USERID}.${GRPID} ${SRCDIR}/samba-2.2.8a )
+( cd ../../../.. ; tar czvf ${SRCDIR}/samba-2.2.8a.tar.gz samba-2.2.8a )
 cp -a *.spec $SPECDIR
 cp -a *.patch smb.* samba.log $SRCDIR
 cd $SPECDIR
