@@ -21,7 +21,7 @@
 
 #include "includes.h"
 
-static char *last_ptr=NULL;
+static const char *last_ptr=NULL;
 
 void set_first_token(char *ptr)
 {
@@ -34,51 +34,56 @@ void set_first_token(char *ptr)
 Based on a routine by GJC@VILLAGE.COM. 
 Extensively modified by Andrew.Tridgell@anu.edu.au
 ****************************************************************************/
-BOOL next_token(char **ptr,char *buff,char *sep, size_t bufsize)
+
+BOOL next_token(const char **ptr,char *buff,const char *sep, size_t bufsize)
 {
-  char *s;
-  BOOL quoted;
-  size_t len=1;
+	const char *s;
+	BOOL quoted;
+	size_t len=1;
 
-  if (!ptr) ptr = &last_ptr;
-  if (!ptr) return(False);
+	if (!ptr)
+		ptr = &last_ptr;
+	if (!ptr)
+		return(False);
 
-  s = *ptr;
+	s = *ptr;
 
-  /* default to simple separators */
-  if (!sep) sep = " \t\n\r";
+	/* default to simple separators */
+	if (!sep)
+		sep = " \t\n\r";
 
-  /* find the first non sep char */
-  while(*s && strchr(sep,*s)) s++;
+	/* find the first non sep char */
+	while(*s && strchr(sep,*s))
+		s++;
 
-  /* nothing left? */
-  if (! *s) return(False);
+	/* nothing left? */
+	if (! *s)
+		return(False);
 
-  /* copy over the token */
-  for (quoted = False; len < bufsize && *s && (quoted || !strchr(sep,*s)); s++)
-    {
-	    if (*s == '\"') {
-		    quoted = !quoted;
-	    } else {
-		    len++;
-		    *buff++ = *s;
-	    }
-    }
+	/* copy over the token */
+	for (quoted = False; len < bufsize && *s && (quoted || !strchr(sep,*s)); s++) {
+		if (*s == '\"') {
+			quoted = !quoted;
+		} else {
+			len++;
+			*buff++ = *s;
+		}
+	}
 
-  *ptr = (*s) ? s+1 : s;  
-  *buff = 0;
-  last_ptr = *ptr;
+	*ptr = (*s) ? s+1 : s;  
+	*buff = 0;
+	last_ptr = *ptr;
 
-  return(True);
+	return(True);
 }
 
 /****************************************************************************
 Convert list of tokens to array; dependent on above routine.
 Uses last_ptr from above - bit of a hack.
 ****************************************************************************/
-char **toktocliplist(int *ctok, char *sep)
+char **toktocliplist(int *ctok, const char *sep)
 {
-  char *s=last_ptr;
+  char *s= (char *)last_ptr;
   int ictok=0;
   char **ret, **iret;
 
@@ -268,7 +273,7 @@ int StrnCaseCmp(const char *s, const char *t, size_t n)
 }
 
 /*******************************************************************
-  compare 2 strings 
+  compare 2 strings - DOS codepage.
 ********************************************************************/
 BOOL strequal(const char *s1, const char *s2)
 {
@@ -278,6 +283,20 @@ BOOL strequal(const char *s1, const char *s2)
   return(StrCaseCmp(s1,s2)==0);
 }
 
+/*******************************************************************
+  compare 2 strings - UNIX codepage.
+********************************************************************/
+BOOL strequal_unix(const char *s1, const char *s2)
+{
+  pstring dos_s1, dos_s2;
+  if (s1 == s2) return(True);
+  if (!s1 || !s2) return(False);
+  
+  pstrcpy(dos_s1, unix_to_dos_static(s1));
+  pstrcpy(dos_s2, unix_to_dos_static(s2));
+  return(StrCaseCmp(dos_s1,dos_s2)==0);
+}
+
 /*******************************************************************
   compare 2 strings up to and including the nth char.
   ******************************************************************/
@@ -303,7 +322,7 @@ BOOL strcsequal(const char *s1,const char *s2)
 /***************************************************************************
 Do a case-insensitive, whitespace-ignoring string compare.
 ***************************************************************************/
-int strwicmp(char *psz1, char *psz2)
+int strwicmp(const char *psz1, const char *psz2)
 {
 	/* if BOTH strings are NULL, return TRUE, if ONE is NULL return */
 	/* appropriate value. */
@@ -437,6 +456,18 @@ void strupper(char *s)
   }
 }
 
+/* Convert a string to upper case, but don't modify it */
+
+char *strupper_static(const char *s)
+{
+	static pstring str;
+
+	pstrcpy(str, s);
+	strupper(str);
+
+	return str;
+}
+
 /*******************************************************************
   convert a string to "normal" form
 ********************************************************************/
@@ -898,18 +929,20 @@ char *safe_strcat(char *dest, const char *src, size_t maxlength)
 
 	if (!src)
 		return dest;
-
+	
 	src_len = strlen(src);
 	dest_len = strlen(dest);
 
 	if (src_len + dest_len > maxlength) {
 		DEBUG(0,("ERROR: string overflow by %d in safe_strcat [%.50s]\n",
-			(int)(src_len + dest_len - maxlength), src));
-		if (dest_len >= maxlength)
-			return dest;
-		src_len = maxlength - dest_len;
+			 (int)(src_len + dest_len - maxlength), src));
+		if (maxlength > dest_len) {
+			memcpy(&dest[dest_len], src, maxlength - dest_len);
+		}
+		dest[maxlength] = 0;
+		return NULL;
 	}
-      
+	
 	memcpy(&dest[dest_len], src, src_len);
 	dest[dest_len + src_len] = 0;
 	return dest;
@@ -1036,7 +1069,7 @@ size_t strhex_to_str(char *p, size_t len, const char *strhex)
 	size_t i;
 	size_t num_chars = 0;
 	unsigned char   lonybble, hinybble;
-	char           *hexchars = "0123456789ABCDEF";
+	const char           *hexchars = "0123456789ABCDEF";
 	char           *p1 = NULL, *p2 = NULL;
 
 	for (i = 0; i < len && strhex[i] != 0; i++)
@@ -1078,7 +1111,7 @@ check if a string is part of a list
 BOOL in_list(char *s,char *list,BOOL casesensitive)
 {
   pstring tok;
-  char *p=list;
+  const char *p=list;
 
   if (!list) return(False);
 
@@ -1163,7 +1196,7 @@ This routine looks for pattern in s and replaces it with
 insert. It may do multiple replacements.
 
 any of " ; ' $ or ` in the insert string are replaced with _
-if len==0 then no length check is performed
+if len==0 then no expansion is permitted.
 ****************************************************************************/
 void string_sub(char *s,const char *pattern,const char *insert, size_t len)
 {
@@ -1177,9 +1210,12 @@ void string_sub(char *s,const char *pattern,const char *insert, size_t len)
 	li = (ssize_t)strlen(insert);
 
 	if (!*pattern) return;
+
+	if (len == 0)
+		len = ls + 1; /* len is number of *bytes* */
 	
 	while (lp <= ls && (p = strstr(s,pattern))) {
-		if (len && (ls + (li-lp) >= len)) {
+		if (ls + (li-lp) >= len) {
 			DEBUG(0,("ERROR: string overflow by %d in string_sub(%.50s, %d)\n", 
 				 (int)(ls + (li-lp) - len),
 				 pattern, (int)len));
@@ -1222,7 +1258,7 @@ void pstring_sub(char *s,const char *pattern,const char *insert)
 /****************************************************************************
 similar to string_sub() but allows for any character to be substituted. 
 Use with caution!
-if len==0 then no length check is performed
+if len==0 then no expansion is permitted.
 ****************************************************************************/
 void all_string_sub(char *s,const char *pattern,const char *insert, size_t len)
 {
@@ -1237,8 +1273,11 @@ void all_string_sub(char *s,const char *pattern,const char *insert, size_t len)
 
 	if (!*pattern) return;
 	
+	if (len == 0)
+		len = ls + 1; /* len is number of *bytes* */
+	
 	while (lp <= ls && (p = strstr(s,pattern))) {
-		if (len && (ls + (li-lp) >= len)) {
+		if (ls + (li-lp) >= len) {
 			DEBUG(0,("ERROR: string overflow by %d in all_string_sub(%.50s, %d)\n", 
 				 (int)(ls + (li-lp) - len),
 				 pattern, (int)len));
@@ -1289,7 +1328,7 @@ void split_at_last_component(char *path, char *front, char sep, char *back)
 /****************************************************************************
 write an octal as a string
 ****************************************************************************/
-char *octal_string(int i)
+const char *octal_string(int i)
 {
 	static char ret[64];
 	if (i == -1) {
@@ -1332,3 +1371,37 @@ char *binary_string(char *buf, int len)
 	s[j] = 0;
 	return s;
 }
+
+#ifndef HAVE_STRNLEN
+/*******************************************************************
+ Some platforms don't have strnlen
+********************************************************************/
+
+ size_t strnlen(const char *s, size_t n)
+{
+	int i;
+	for (i=0; s[i] && i<n; i++)
+		/* noop */ ;
+	return i;
+}
+#endif
+
+#ifndef HAVE_STRNDUP
+/*******************************************************************
+ Some platforms don't have strndup.
+********************************************************************/
+
+ char *strndup(const char *s, size_t n)
+{
+	char *ret;
+
+	n = strnlen(s, n);
+	ret = malloc(n+1);
+	if (!ret)
+		return NULL;
+	memcpy(ret, s, n);
+	ret[n] = 0;
+
+	return ret;
+}
+#endif
