@@ -23,6 +23,24 @@
 
 #include "winbindd.h"
 
+/*******************************************************************
+ Duplicate a UNISTR2 string into a UNIX codepage null terminated char*
+ using a talloc context
+********************************************************************/
+
+static char *unistr2_tdup(TALLOC_CTX *ctx, const UNISTR2 *str)
+{
+	char *s;
+	int maxlen = (str->uni_str_len+1)*4;
+	if (!str->buffer)
+		return NULL;
+	s = (char *)talloc(ctx, maxlen); /* convervative */
+	if (!s)
+		return NULL;
+	unistr2_to_unix(s, str, maxlen);
+	return s;
+}
+
 /* Query display info for a domain.  This returns enough information plus a
    bit extra to give an overview of domain users for the User Manager
    application. */
@@ -32,24 +50,29 @@ static NTSTATUS query_user_list(struct winbindd_domain *domain,
 			       WINBIND_USERINFO **info)
 {
 	CLI_POLICY_HND *hnd;
-	NTSTATUS result = NT_STATUS_UNSUCCESSFUL;
+	NTSTATUS result;
 	POLICY_HND dom_pol;
 	BOOL got_dom_pol = False;
 	uint32 des_access = SEC_RIGHTS_MAXIMUM_ALLOWED;
-	int i;
+	int i, loop_count = 0;
+	int retry;
 
 	*num_entries = 0;
 	*info = NULL;
 
-	/* Get sam handle */
+	retry = 0;
+	do {
+		/* Get sam handle */
 
-	if (!(hnd = cm_get_sam_handle(domain->name)))
-		goto done;
+		if (!NT_STATUS_IS_OK(result = cm_get_sam_handle(domain->name, &hnd)))
+			goto done;
 
-	/* Get domain handle */
+		/* Get domain handle */
 
-	result = cli_samr_open_domain(hnd->cli, mem_ctx, &hnd->pol,
+		result = cli_samr_open_domain(hnd->cli, mem_ctx, &hnd->pol,
 					des_access, &domain->sid, &dom_pol);
+	} while (!NT_STATUS_IS_OK(result) && (retry++ < 1) &&
+			hnd && hnd->cli && hnd->cli->fd == -1);
 
 	if (!NT_STATUS_IS_OK(result))
 		goto done;
@@ -60,7 +83,7 @@ static NTSTATUS query_user_list(struct winbindd_domain *domain,
 	do {
 		SAM_DISPINFO_CTR ctr;
 		SAM_DISPINFO_1 info1;
-		uint32 count = 0, start=i;
+		uint32 count = 0, start=i, max_entries, max_size;
 		int j;
 		TALLOC_CTX *ctx2;
 
@@ -72,10 +95,15 @@ static NTSTATUS query_user_list(struct winbindd_domain *domain,
 			goto done;
 		}
 		
+		get_query_dispinfo_params(
+			loop_count, &max_entries, &max_size);
+
 		/* Query display info level 1 */
-		result = cli_samr_query_dispinfo(hnd->cli, ctx2,
-						 &dom_pol, &start, 1,
-						 &count, 0xFFFF, &ctr);
+		result = cli_samr_query_dispinfo(
+			hnd->cli, ctx2, &dom_pol, &start, 1, &count, 
+			max_entries, max_size, &ctr);
+
+		loop_count++;
 
 		if (!NT_STATUS_IS_OK(result) && 
 		    !NT_STATUS_EQUAL(result, STATUS_MORE_ENTRIES)) break;
@@ -92,6 +120,7 @@ static NTSTATUS query_user_list(struct winbindd_domain *domain,
 		}
 
 		for (j=0;j<count;i++, j++) {
+			/* unistr2_tdup converts to UNIX charset. */
 			(*info)[i].acct_name = unistr2_tdup(mem_ctx, &info1.str[j].uni_acct_name);
 			(*info)[i].full_name = unistr2_tdup(mem_ctx, &info1.str[j].uni_full_name);
 			(*info)[i].user_rid = info1.sam[j].rid_user;
@@ -116,7 +145,8 @@ static NTSTATUS query_user_list(struct winbindd_domain *domain,
 	return result;
 }
 
-/* list all domain groups */
+/* List all domain groups */
+
 static NTSTATUS enum_dom_groups(struct winbindd_domain *domain,
 				TALLOC_CTX *mem_ctx,
 				uint32 *num_entries, 
@@ -125,35 +155,40 @@ static NTSTATUS enum_dom_groups(struct winbindd_domain *domain,
 	uint32 des_access = SEC_RIGHTS_MAXIMUM_ALLOWED;
 	CLI_POLICY_HND *hnd;
 	POLICY_HND dom_pol;
-	NTSTATUS status;
+	NTSTATUS result;
+	uint32 start = 0;
+	int retry;
 
 	*num_entries = 0;
 	*info = NULL;
 
-	if (!(hnd = cm_get_sam_handle(domain->name))) {
-		return NT_STATUS_UNSUCCESSFUL;
-	}
+	retry = 0;
+	do {
+		if (!NT_STATUS_IS_OK(result = cm_get_sam_handle(domain->name, &hnd)))
+			return result;
 
-	status = cli_samr_open_domain(hnd->cli, mem_ctx,
+		result = cli_samr_open_domain(hnd->cli, mem_ctx,
 				      &hnd->pol, des_access, &domain->sid, &dom_pol);
-	if (!NT_STATUS_IS_OK(status)) {
-		return status;
-	}
+	} while (!NT_STATUS_IS_OK(result) && (retry++ < 1) &&
+			hnd && hnd->cli && hnd->cli->fd == -1);
+
+	if (!NT_STATUS_IS_OK(result))
+		return result;
 
 	do {
 		struct acct_info *info2 = NULL;
-		uint32 count = 0, start = *num_entries;
+		uint32 count = 0;
 		TALLOC_CTX *mem_ctx2;
 
 		mem_ctx2 = talloc_init_named("enum_dom_groups[rpc]");
 
-		status = cli_samr_enum_dom_groups(hnd->cli, mem_ctx2, &dom_pol,
-						  &start,
-						  0xFFFF, /* buffer size? */
-						  &info2, &count);
+		/* This call updates 'start' */
+		result = cli_samr_enum_dom_groups(
+			hnd->cli, mem_ctx2, &dom_pol, &start,
+			0xFFFF, /* buffer size? */ &info2, &count);
 
-		if (!NT_STATUS_IS_OK(status) && 
-		    !NT_STATUS_EQUAL(status, STATUS_MORE_ENTRIES)) {
+		if (!NT_STATUS_IS_OK(result) && 
+		    !NT_STATUS_EQUAL(result, STATUS_MORE_ENTRIES)) {
 			talloc_destroy(mem_ctx2);
 			break;
 		}
@@ -169,11 +204,11 @@ static NTSTATUS enum_dom_groups(struct winbindd_domain *domain,
 		memcpy(&(*info)[*num_entries], info2, count*sizeof(*info2));
 		(*num_entries) += count;
 		talloc_destroy(mem_ctx2);
-	} while (NT_STATUS_EQUAL(status, STATUS_MORE_ENTRIES));
+	} while (NT_STATUS_EQUAL(result, STATUS_MORE_ENTRIES));
 
 	cli_samr_close(hnd->cli, mem_ctx, &dom_pol);
 
-	return status;
+	return result;
 }
 
 /* convert a single name to a sid in a domain */
@@ -184,21 +219,17 @@ static NTSTATUS name_to_sid(struct winbindd_domain *domain,
 {
 	TALLOC_CTX *mem_ctx;
 	CLI_POLICY_HND *hnd;
-	NTSTATUS status;
+	NTSTATUS result;
 	DOM_SID *sids = NULL;
 	uint32 *types = NULL;
 	const char *full_name;
+	int retry;
 
 	if (!(mem_ctx = talloc_init_named("name_to_sid[rpc] for [%s]\\[%s]", domain->name, name))) {
 		DEBUG(0, ("talloc_init failed!\n"));
 		return NT_STATUS_NO_MEMORY;
 	}
         
-	if (!(hnd = cm_get_lsa_handle(domain->name))) {
-		talloc_destroy(mem_ctx);
-		return NT_STATUS_UNSUCCESSFUL;
-	}
-        
 	full_name = talloc_asprintf(mem_ctx, "%s\\%s", domain->name, name);
 	
 	if (!full_name) {
@@ -207,18 +238,27 @@ static NTSTATUS name_to_sid(struct winbindd_domain *domain,
 		return NT_STATUS_NO_MEMORY;
 	}
 
-	status = cli_lsa_lookup_names(hnd->cli, mem_ctx, &hnd->pol, 1, 
+	retry = 0;
+	do {
+		if (!NT_STATUS_IS_OK(result = cm_get_lsa_handle(domain->name, &hnd))) {
+			talloc_destroy(mem_ctx);
+			return NT_STATUS_UNSUCCESSFUL;
+		}
+        
+		result = cli_lsa_lookup_names(hnd->cli, mem_ctx, &hnd->pol, 1, 
 				      &full_name, &sids, &types);
+	} while (!NT_STATUS_IS_OK(result) && (retry++ < 1) &&
+			hnd && hnd->cli && hnd->cli->fd == -1);
         
 	/* Return rid and type if lookup successful */
 
-	if (NT_STATUS_IS_OK(status)) {
+	if (NT_STATUS_IS_OK(result)) {
 		sid_copy(sid, &sids[0]);
 		*type = types[0];
 	}
 
 	talloc_destroy(mem_ctx);
-	return status;
+	return result;
 }
 
 /*
@@ -234,15 +274,20 @@ static NTSTATUS sid_to_name(struct winbindd_domain *domain,
 	char **domains;
 	char **names;
 	uint32 *types;
-	NTSTATUS status;
-
-	if (!(hnd = cm_get_lsa_handle(domain->name)))
-		return NT_STATUS_UNSUCCESSFUL;
+	NTSTATUS result;
+	int retry;
+		
+	retry = 0;
+	do {
+		if (!NT_STATUS_IS_OK(result = cm_get_lsa_handle(domain->name, &hnd)))
+			return NT_STATUS_UNSUCCESSFUL;
         
-	status = cli_lsa_lookup_sids(hnd->cli, mem_ctx, &hnd->pol,
+		result = cli_lsa_lookup_sids(hnd->cli, mem_ctx, &hnd->pol,
 				     1, sid, &domains, &names, &types);
+	} while (!NT_STATUS_IS_OK(result) && (retry++ < 1) &&
+			hnd && hnd->cli && hnd->cli->fd == -1);
 
-	if (NT_STATUS_IS_OK(status)) {
+	if (NT_STATUS_IS_OK(result)) {
 		*type = types[0];
 		*name = names[0];
 		DEBUG(5,("Mapped sid to [%s]\\[%s]\n", domains[0], *name));
@@ -253,7 +298,8 @@ static NTSTATUS sid_to_name(struct winbindd_domain *domain,
 			return NT_STATUS_UNSUCCESSFUL;
 		}
 	}
-	return status;
+
+	return result;
 }
 
 /* Lookup user information from a rid or username. */
@@ -267,16 +313,20 @@ static NTSTATUS query_user(struct winbindd_domain *domain,
 	POLICY_HND dom_pol, user_pol;
 	BOOL got_dom_pol = False, got_user_pol = False;
 	SAM_USERINFO_CTR *ctr;
+	int retry;
 
-	/* Get sam handle */
-	if (!(hnd = cm_get_sam_handle(domain->name)))
-		goto done;
+	retry = 0;
+	do {
+		/* Get sam handle */
+		if (!NT_STATUS_IS_OK(result = cm_get_sam_handle(domain->name, &hnd)))
+			goto done;
 
-	/* Get domain handle */
-
-	result = cli_samr_open_domain(hnd->cli, mem_ctx, &hnd->pol,
+		/* Get domain handle */
+		result = cli_samr_open_domain(hnd->cli, mem_ctx, &hnd->pol,
 				      SEC_RIGHTS_MAXIMUM_ALLOWED, 
 				      &domain->sid, &dom_pol);
+	} while (!NT_STATUS_IS_OK(result) && (retry++ < 1) &&
+			hnd && hnd->cli && hnd->cli->fd == -1);
 
 	if (!NT_STATUS_IS_OK(result))
 		goto done;
@@ -296,9 +346,13 @@ static NTSTATUS query_user(struct winbindd_domain *domain,
 	result = cli_samr_query_userinfo(hnd->cli, mem_ctx, &user_pol, 
 					 0x15, &ctr);
 
+	if (!NT_STATUS_IS_OK(result))
+		goto done;
+
 	cli_samr_close(hnd->cli, mem_ctx, &user_pol);
 	got_user_pol = False;
 
+	user_info->user_rid = user_rid;
 	user_info->group_rid = ctr->info.id21->group_rid;
 	user_info->acct_name = unistr2_tdup(mem_ctx, 
 					    &ctr->info.id21->uni_user_name);
@@ -323,23 +377,28 @@ static NTSTATUS lookup_usergroups(struct winbindd_domain *domain,
 				  uint32 *num_groups, uint32 **user_gids)
 {
 	CLI_POLICY_HND *hnd;
-	NTSTATUS result = NT_STATUS_UNSUCCESSFUL;
+	NTSTATUS result;
 	POLICY_HND dom_pol, user_pol;
 	uint32 des_access = SEC_RIGHTS_MAXIMUM_ALLOWED;
 	BOOL got_dom_pol = False, got_user_pol = False;
 	DOM_GID *user_groups;
 	int i;
+	int retry;
 
 	*num_groups = 0;
 	*user_gids = NULL;
 
-	/* Get sam handle */
-	if (!(hnd = cm_get_sam_handle(domain->name)))
-		goto done;
+	retry = 0;
+	do {
+		/* Get sam handle */
+		if (!NT_STATUS_IS_OK(result = cm_get_sam_handle(domain->name, &hnd)))
+			goto done;
 
-	/* Get domain handle */
-	result = cli_samr_open_domain(hnd->cli, mem_ctx, &hnd->pol,
-					des_access, &domain->sid, &dom_pol);
+		/* Get domain handle */
+		result = cli_samr_open_domain(hnd->cli, mem_ctx, &hnd->pol,
+				      des_access, &domain->sid, &dom_pol);
+	} while (!NT_STATUS_IS_OK(result) && (retry++ < 1) &&
+			hnd && hnd->cli && hnd->cli->fd == -1);
 
 	if (!NT_STATUS_IS_OK(result))
 		goto done;
@@ -387,23 +446,26 @@ static NTSTATUS lookup_groupmem(struct winbindd_domain *domain,
 				uint32 **name_types)
 {
         CLI_POLICY_HND *hnd;
-        NTSTATUS result = NT_STATUS_UNSUCCESSFUL;
+        NTSTATUS result;
         uint32 i, total_names = 0;
         POLICY_HND dom_pol, group_pol;
         uint32 des_access = SEC_RIGHTS_MAXIMUM_ALLOWED;
         BOOL got_dom_pol = False, got_group_pol = False;
+	int retry;
 
 	*num_names = 0;
 
-        /* Get sam handle */
+	retry = 0;
+	do {
+	        /* Get sam handle */
+		if (!NT_STATUS_IS_OK(result = cm_get_sam_handle(domain->name, &hnd)))
+			goto done;
 
-        if (!(hnd = cm_get_sam_handle(domain->name)))
-                goto done;
-
-        /* Get domain handle */
-
-        result = cli_samr_open_domain(hnd->cli, mem_ctx, &hnd->pol,
-                                      des_access, &domain->sid, &dom_pol);
+		/* Get domain handle */
+		result = cli_samr_open_domain(hnd->cli, mem_ctx, &hnd->pol,
+				des_access, &domain->sid, &dom_pol);
+	} while (!NT_STATUS_IS_OK(result) && (retry++ < 1) &&
+			hnd && hnd->cli && hnd->cli->fd == -1);
 
         if (!NT_STATUS_IS_OK(result))
                 goto done;
@@ -483,6 +545,94 @@ static NTSTATUS lookup_groupmem(struct winbindd_domain *domain,
 }
 
 /* find the sequence number for a domain */
+
+#ifdef WITH_HORRIBLE_LDAP_NATIVE_MODE_HACK
+#include <ldap.h>
+
+static SIG_ATOMIC_T gotalarm;
+
+/***************************************************************
+ Signal function to tell us we timed out.
+****************************************************************/
+
+static void gotalarm_sig(void)
+{
+	gotalarm = 1;
+}
+
+static LDAP *ldap_open_with_timeout(const char *server, int port, unsigned int to)
+{
+	LDAP *ldp = NULL;
+
+	/* Setup timeout */
+	gotalarm = 0;
+	CatchSignal(SIGALRM, SIGNAL_CAST gotalarm_sig);
+	alarm(to);
+	/* End setup timeout. */
+
+	ldp = ldap_open(server, port);
+
+	/* Teardown timeout. */
+	CatchSignal(SIGALRM, SIGNAL_CAST SIG_IGN);
+	alarm(0);
+
+	return ldp;
+}
+
+int get_ldap_seq(const char *server, uint32 *seq)
+{
+	int ret = -1;
+	struct timeval to;
+	char *attrs[] = {"highestCommittedUSN", NULL};
+	LDAPMessage *res = NULL;
+	char **values = NULL;
+	LDAP *ldp = NULL;
+
+	*seq = DOM_SEQUENCE_NONE;
+
+	/*
+	 * 10 second timeout on open. This is needed as the search timeout
+	 * doesn't seem to apply to doing an open as well. JRA.
+	 */
+
+	if ((ldp = ldap_open_with_timeout(server, LDAP_PORT, 10)) == NULL)
+		return -1;
+
+#if 0
+	/* As per tridge comment this doesn't seem to be needed. JRA */
+	if ((err = ldap_simple_bind_s(ldp, NULL, NULL)) != 0)
+		goto done;
+#endif
+
+	/* Timeout if no response within 20 seconds. */
+	to.tv_sec = 10;
+	to.tv_usec = 0;
+
+	if (ldap_search_st(ldp, "", LDAP_SCOPE_BASE, "(objectclass=*)", &attrs[0], 0, &to, &res))
+		goto done;
+
+	if (ldap_count_entries(ldp, res) != 1)
+		goto done;
+
+	values = ldap_get_values(ldp, res, "highestCommittedUSN");
+	if (!values || !values[0])
+		goto done;
+
+	*seq = atoi(values[0]);
+	ret = 0;
+
+  done:
+
+	if (values)
+		ldap_value_free(values);
+	if (res)
+		ldap_msgfree(res);
+	if (ldp)
+		ldap_unbind(ldp);
+	return ret;
+}
+#endif /* WITH_HORRIBLE_LDAP_NATIVE_MODE_HACK */
+
 static NTSTATUS sequence_number(struct winbindd_domain *domain, uint32 *seq)
 {
 	TALLOC_CTX *mem_ctx;
@@ -494,21 +644,38 @@ static NTSTATUS sequence_number(struct winbindd_domain *domain, uint32 *seq)
 	POLICY_HND dom_pol;
 	BOOL got_dom_pol = False;
 	uint32 des_access = SEC_RIGHTS_MAXIMUM_ALLOWED;
+	int retry;
 
 	*seq = DOM_SEQUENCE_NONE;
 
 	if (!(mem_ctx = talloc_init_named("sequence_number[rpc]")))
 		return NT_STATUS_NO_MEMORY;
 
-	/* Get sam handle */
+	retry = 0;
+	do {
+		/* Get sam handle */
+		if (!NT_STATUS_IS_OK(result = cm_get_sam_handle(domain->name, &hnd)))
+			goto done;
 
-	if (!(hnd = cm_get_sam_handle(domain->name)))
-		goto done;
+#ifdef WITH_HORRIBLE_LDAP_NATIVE_MODE_HACK
+		if (get_ldap_seq( inet_ntoa(hnd->cli->dest_ip), seq) == 0) {
+			result = NT_STATUS_OK;
+			seqnum = *seq;
+			DEBUG(10,("domain_sequence_number: LDAP for domain %s is %u\n",
+					domain->name, (unsigned)seqnum ));
+			goto done;
+		}
 
-	/* Get domain handle */
+		DEBUG(10,("domain_sequence_number: failed to get LDAP sequence number (%u) for domain %s\n",
+		(unsigned)seqnum, domain->name ));
 
-	result = cli_samr_open_domain(hnd->cli, mem_ctx, &hnd->pol, 
+#endif /* WITH_HORRIBLE_LDAP_NATIVE_MODE_HACK */
+
+		/* Get domain handle */
+		result = cli_samr_open_domain(hnd->cli, mem_ctx, &hnd->pol, 
 				      des_access, &domain->sid, &dom_pol);
+	} while (!NT_STATUS_IS_OK(result) && (retry++ < 1) &&
+			hnd && hnd->cli && hnd->cli->fd == -1);
 
 	if (!NT_STATUS_IS_OK(result))
 		goto done;
@@ -522,6 +689,9 @@ static NTSTATUS sequence_number(struct winbindd_domain *domain, uint32 *seq)
 
 	if (NT_STATUS_IS_OK(result)) {
 		seqnum = ctr.info.inf2.seq_num;
+		seqnum += ctr.info.inf2.num_domain_usrs;
+		seqnum += ctr.info.inf2.num_domain_grps;
+		seqnum += ctr.info.inf2.num_local_grps;
 		DEBUG(10,("domain_sequence_number: for domain %s is %u\n", domain->name, (unsigned)seqnum ));
 	} else {
 		DEBUG(10,("domain_sequence_number: failed to get sequence number (%u) for domain %s\n",
@@ -548,17 +718,22 @@ static NTSTATUS trusted_domains(struct winbindd_domain *domain,
 				DOM_SID **dom_sids)
 {
 	CLI_POLICY_HND *hnd;
-	NTSTATUS result = NT_STATUS_UNSUCCESSFUL;
+	NTSTATUS result;
 	uint32 enum_ctx = 0;
+	int retry;
 
 	*num_domains = 0;
 
-	if (!(hnd = cm_get_lsa_handle(lp_workgroup())))
-		goto done;
+	retry = 0;
+	do {
+		if (!NT_STATUS_IS_OK(result = cm_get_lsa_handle(lp_workgroup(), &hnd)))
+			goto done;
 
-	result = cli_lsa_enum_trust_dom(hnd->cli, mem_ctx,
+		result = cli_lsa_enum_trust_dom(hnd->cli, mem_ctx,
 					&hnd->pol, &enum_ctx, num_domains, 
 					names, dom_sids);
+	} while (!NT_STATUS_IS_OK(result) && (retry++ < 1) &&
+			hnd && hnd->cli && hnd->cli->fd == -1);
 done:
 	return result;
 }
@@ -566,24 +741,29 @@ done:
 /* find the domain sid for a domain */
 static NTSTATUS domain_sid(struct winbindd_domain *domain, DOM_SID *sid)
 {
-	NTSTATUS status = NT_STATUS_UNSUCCESSFUL;
+	NTSTATUS result;
 	TALLOC_CTX *mem_ctx;
 	CLI_POLICY_HND *hnd;
 	fstring level5_dom;
+	int retry;
 
 	if (!(mem_ctx = talloc_init_named("domain_sid[rpc]")))
 		return NT_STATUS_NO_MEMORY;
 
-	/* Get sam handle */
-	if (!(hnd = cm_get_lsa_handle(domain->name)))
-		goto done;
+	retry = 0;
+	do {
+		/* Get lsa handle */
+		if (!NT_STATUS_IS_OK(result = cm_get_lsa_handle(domain->name, &hnd)))
+			goto done;
 
-	status = cli_lsa_query_info_policy(hnd->cli, mem_ctx,
-					   &hnd->pol, 0x05, level5_dom, sid);
+		result = cli_lsa_query_info_policy(hnd->cli, mem_ctx,
+				&hnd->pol, 0x05, level5_dom, sid);
 
+	} while (!NT_STATUS_IS_OK(result) && (retry++ < 1) &&
+			hnd && hnd->cli && hnd->cli->fd == -1);
 done:
 	talloc_destroy(mem_ctx);
-	return status;
+	return result;
 }
 
 /* the rpc backend methods are exposed via this structure */
