@@ -251,6 +251,8 @@ static void endsmbfilepwent(void *vp, int *lock_depth)
 {
   FILE *fp = (FILE *)vp;
 
+  if (!fp)
+	return;
   pw_file_unlock(fileno(fp), lock_depth);
   fclose(fp);
   DEBUG(7, ("endsmbfilepwent_internal: closed password file.\n"));
@@ -1236,15 +1238,15 @@ static BOOL build_sam_account(SAM_ACCOUNT *sam_pass, struct smb_passwd *pw_buf)
 	        sam_logon_in_ssb = True;
 
 	        pstrcpy(str, lp_logon_script());
-       		standard_sub_advanced(-1, pw_buf->smb_name, "", gid, str);
+       		standard_sub_advanced(-1, pw_buf->smb_name, "", gid, str,sizeof(str));
 		pdb_set_logon_script(sam_pass, str, False);
 
 	        pstrcpy(str, lp_logon_path());
-       		standard_sub_advanced(-1, pw_buf->smb_name, "", gid, str);
+       		standard_sub_advanced(-1, pw_buf->smb_name, "", gid, str,sizeof(str));
 		pdb_set_profile_path(sam_pass, str, False);
 
 	        pstrcpy(str, lp_logon_home());
-        	standard_sub_advanced(-1, pw_buf->smb_name, "", gid, str);
+        	standard_sub_advanced(-1, pw_buf->smb_name, "", gid, str,sizeof(str));
 		pdb_set_homedir(sam_pass, str, False);
  		
 		sam_logon_in_ssb = False;
@@ -1333,7 +1335,7 @@ BOOL pdb_getsampwent(SAM_ACCOUNT *user)
  call getpwnam() for unix account information until we have found
  the correct entry
  ***************************************************************/
-BOOL pdb_getsampwnam(SAM_ACCOUNT *sam_acct, char *username)
+BOOL pdb_getsampwnam(SAM_ACCOUNT *sam_acct, const char *username)
 {
 	struct smb_passwd *smb_pw;
 	void *fp = NULL;
@@ -1473,7 +1475,7 @@ BOOL pdb_update_sam_account(SAM_ACCOUNT *sampass, BOOL override)
 	return True;
 }
 
-BOOL pdb_delete_sam_account (char* username)
+BOOL pdb_delete_sam_account (const char* username)
 {
 	return del_smbfilepwd_entry(username);
 }
