@@ -85,7 +85,7 @@ NTSTATUS _net_logon_ctrl2(pipes_struct *p, NET_Q_LOGON_CTRL2 *q_u, NET_R_LOGON_C
 	uint32 pdc_connection_status = 0x0;
 	uint32 logon_attempts = 0x0;
 	uint32 tc_status = ERROR_NO_LOGON_SERVERS;
-	char *trusted_domain = "test_domain";
+	const char *trusted_domain = "test_domain";
 
         DEBUG(0, ("*** net long ctrl2 %d, %d, %d\n",
                   q_u->function_code, q_u->query_level, q_u->switch_value));
@@ -109,7 +109,7 @@ NTSTATUS _net_logon_ctrl2(pipes_struct *p, NET_Q_LOGON_CTRL2 *q_u, NET_R_LOGON_C
 
 NTSTATUS _net_trust_dom_list(pipes_struct *p, NET_Q_TRUST_DOM_LIST *q_u, NET_R_TRUST_DOM_LIST *r_u)
 {
-	char *trusted_domain = "test_domain";
+	const char *trusted_domain = "test_domain";
 	uint32 num_trust_domains = 1;
 
 	DEBUG(6,("_net_trust_dom_list: %d\n", __LINE__));
@@ -223,7 +223,7 @@ NTSTATUS _net_req_chal(pipes_struct *p, NET_Q_REQ_CHAL *q_u, NET_R_REQ_CHAL *r_u
 
 		/* from client / server challenges and md4 password, generate sess key */
 		cred_session_key(&p->dc.clnt_chal, &p->dc.srv_chal,
-				 (char *)p->dc.md4pw, p->dc.sess_key);
+				(uchar *)p->dc.md4pw, p->dc.sess_key);
 
 		/* Save the machine account name. */
 		fstrcpy(p->dc.mach_acct, mach_acct);
