@@ -227,7 +227,7 @@ static int setup_fake_group_map(GROUP_MAP **ret_map)
 
 	static_map[0].gid = (gid_t)-1;
 	sid_copy(&static_map[0].sid, &global_sam_sid);
-	sid_append_rid(&static_map[1].sid, DOMAIN_GROUP_RID_ADMINS);
+	sid_append_rid(&static_map[0].sid, DOMAIN_GROUP_RID_ADMINS);
 	static_map[0].sid_name_use = SID_NAME_DOM_GRP;
 	fstrcpy(static_map[0].nt_name, "Domain Admins");
 	fstrcpy(static_map[0].comment, "Administrators for the domain");
@@ -657,10 +657,10 @@ static void make_group_sam_entry_list(TALLOC_CTX *ctx, SAM_ENTRY **sam_pp, UNIST
 		/*
 		 * JRA. I think this should include the null. TNG does not.
 		 */
-		int len = strlen(grp[i].name)+1;
+		int len = strlen(unix_to_dos_static(grp[i].name))+1;
 
 		init_sam_entry(&sam[i], len, grp[i].rid);
-		init_unistr2(&uni_name[i], grp[i].name, len);
+		init_unistr2(&uni_name[i], unix_to_dos_static(grp[i].name), len);
 	}
 
 	*sam_pp = sam;
@@ -685,7 +685,7 @@ static NTSTATUS get_group_alias_entries(DOMAIN_GRP *d_grp, DOM_SID *sid, uint32
 
 	/* well-known aliases */
 	if (strequal(sid_str, "S-1-5-32")) {
-		char *alias_name;
+		const char *alias_name;
 		while (!lp_hide_local_users() &&
 				num_entries < max_entries && 
 				((alias_name = builtin_alias_rids[num_entries].name) != NULL)) {
