@@ -26,8 +26,8 @@
 #include "rpcclient.h"
 
 struct table_node {
-	char 	*long_archi;
-	char 	*short_archi;
+	const char 	*long_archi;
+	const char 	*short_archi;
 	int	version;
 };
  
@@ -45,7 +45,7 @@ struct table_node archi_table[]= {
 function to do the mapping between the long architecture name and
 the short one.
 ****************************************************************************/
-BOOL get_short_archi(char *short_archi, char *long_archi)
+BOOL get_short_archi(char *short_archi, const char *long_archi)
 {
         int i=-1;
 
@@ -969,7 +969,7 @@ void set_drv_info_3_env (DRIVER_INFO_3 *info, const char *arch)
  wrapper for strtok to get the next parameter from a delimited list.
  Needed to handle the empty parameter string denoted by "NULL"
  *************************************************************************/
-static char* get_driver_3_param (char* str, char* delim, UNISTR* dest)
+static char* get_driver_3_param (char* str, const char* delim, UNISTR* dest)
 {
 	char	*ptr;
 
@@ -1256,16 +1256,21 @@ static NTSTATUS cmd_spoolss_deletedriver(struct cli_state *cli,
 		result = cli_spoolss_deleteprinterdriver(
 			cli, mem_ctx, archi_table[i].long_archi, argv[1]);
 
-		if (!W_ERROR_IS_OK(result)) {
-			printf ("Failed to remove driver %s for arch [%s] - error 0x%x!\n", 
-				argv[1], archi_table[i].long_archi, 
-				W_ERROR_V(result));
-		} else
+		if ( !W_ERROR_IS_OK(result) ) {
+			if ( !W_ERROR_EQUAL(result, WERR_UNKNOWN_PRINTER_DRIVER) ) {
+				printf ("Failed to remove driver %s for arch [%s] - error 0x%x!\n", 
+					argv[1], archi_table[i].long_archi, 
+					W_ERROR_V(result));
+			}
+		} 
+		else 
+		{
 			printf ("Driver %s removed for arch [%s].\n", argv[1], 
 				archi_table[i].long_archi);
+		}
 	}
 		
-	return W_ERROR_IS_OK(result) ? NT_STATUS_OK : NT_STATUS_UNSUCCESSFUL;
+	return W_ERROR_IS_OK(result) || W_ERROR_EQUAL(result, WERR_UNKNOWN_PRINTER_DRIVER) ? NT_STATUS_OK : NT_STATUS_UNSUCCESSFUL;
 }
 
 static NTSTATUS cmd_spoolss_getprintprocdir(struct cli_state *cli, 
