@@ -55,6 +55,7 @@ struct vfs_ops default_vfs_ops = {
 	vfswrap_read,
 	vfswrap_write,
 	vfswrap_lseek,
+	 vfswrap_sendfile,
 	vfswrap_rename,
 	vfswrap_fsync,
 	vfswrap_stat,
@@ -345,13 +346,13 @@ ssize_t vfs_write_data(files_struct *fsp,const char *buffer,size_t N)
  Returns 0 on success, -1 on failure.
 ****************************************************************************/
 
-int vfs_allocate_file_space(files_struct *fsp, SMB_OFF_T len)
+int vfs_allocate_file_space(files_struct *fsp, SMB_BIG_UINT len)
 {
 	int ret;
 	SMB_STRUCT_STAT st;
 	connection_struct *conn = fsp->conn;
 	struct vfs_ops *vfs_ops = &conn->vfs_ops;
-	SMB_OFF_T space_avail;
+	SMB_BIG_UINT space_avail;
 	SMB_BIG_UINT bsize,dfree,dsize;
 
 	release_level_2_oplocks_on_change(fsp);
@@ -362,21 +363,26 @@ int vfs_allocate_file_space(files_struct *fsp, SMB_OFF_T len)
 
 	DEBUG(10,("vfs_allocate_file_space: file %s, len %.0f\n", fsp->fsp_name, (double)len ));
 
+	if (((SMB_OFF_T)len) < 0) {
+		DEBUG(0,("vfs_allocate_file_space: %s negative len requested.\n", fsp->fsp_name ));
+		return -1;
+	}
+
 	ret = vfs_fstat(fsp,fsp->fd,&st);
 	if (ret == -1)
 		return ret;
 
-	if (len == st.st_size)
+	if (len == (SMB_BIG_UINT)st.st_size)
 		return 0;
 
-	if (len < st.st_size) {
+	if (len < (SMB_BIG_UINT)st.st_size) {
 		/* Shrink - use ftruncate. */
 
 		DEBUG(10,("vfs_allocate_file_space: file %s, shrink. Current size %.0f\n",
 				fsp->fsp_name, (double)st.st_size ));
 
 		flush_write_cache(fsp, SIZECHANGE_FLUSH);
-		if ((ret = vfs_ops->ftruncate(fsp, fsp->fd, len)) != -1) {
+		if ((ret = vfs_ops->ftruncate(fsp, fsp->fd, (SMB_OFF_T)len)) != -1) {
 			set_filelen_write_cache(fsp, len);
 		}
 		return ret;
@@ -389,10 +395,10 @@ int vfs_allocate_file_space(files_struct *fsp, SMB_OFF_T len)
 
 	len -= st.st_size;
 	len /= 1024; /* Len is now number of 1k blocks needed. */
-	space_avail = (SMB_OFF_T)conn->vfs_ops.disk_free(conn,fsp->fsp_name,False,&bsize,&dfree,&dsize);
+	space_avail = conn->vfs_ops.disk_free(conn,fsp->fsp_name,False,&bsize,&dfree,&dsize);
 
-	DEBUG(10,("vfs_allocate_file_space: file %s, grow. Current size %.0f, needed blocks = %lu, space avail = %lu\n",
-			fsp->fsp_name, (double)st.st_size, (unsigned long)len, (unsigned long)space_avail ));
+	DEBUG(10,("vfs_allocate_file_space: file %s, grow. Current size %.0f, needed blocks = %.0f, space avail = %.0f\n",
+			fsp->fsp_name, (double)st.st_size, (double)len, (double)space_avail ));
 
 	if (len > space_avail) {
 		errno = ENOSPC;
@@ -699,6 +705,63 @@ char *vfs_GetWd(connection_struct *conn, char *path)
 	return (path);
 }
 
+
+/* check if the file 'nmae' is a symlink, in that case check that it point to
+   a file that reside under the 'dir' tree */
+
+static BOOL readlink_check(connection_struct *conn, char *dir, char *name)
+{
+	BOOL ret = True;
+	pstring flink;
+	pstring cleanlink;
+	pstring savedir;
+	pstring realdir;
+	size_t reallen;
+
+	if (!vfs_GetWd(conn, savedir)) {
+		DEBUG(0,("couldn't vfs_GetWd for %s %s\n", name, dir));
+		return False;
+	}
+
+	if (vfs_ChDir(conn, dir) != 0) {
+		DEBUG(0,("couldn't vfs_ChDir to %s\n", dir));
+		return False;
+	}
+
+	if (!vfs_GetWd(conn, realdir)) {
+		DEBUG(0,("couldn't vfs_GetWd for %s\n", dir));
+		vfs_ChDir(conn, savedir);
+		return(False);
+	}
+	
+	reallen = strlen(realdir);
+	if (realdir[reallen -1] == '/') {
+		reallen--;
+		realdir[reallen] = 0;
+	}
+
+	if (conn->vfs_ops.readlink(conn, name, flink, sizeof(pstring) -1) != -1) {
+		DEBUG(3,("reduce_name: file path name %s is a symlink\nChecking it's path\n", name));
+		if (*flink == '/') {
+			pstrcpy(cleanlink, flink);
+		} else {
+			pstrcpy(cleanlink, realdir);
+			pstrcat(cleanlink, "/");
+			pstrcat(cleanlink, flink);
+		}
+		unix_clean_name(cleanlink);
+
+		if (strncmp(cleanlink, realdir, reallen) != 0) {
+			DEBUG(2,("Bad access attempt? s=%s dir=%s newname=%s l=%d\n", name, realdir, cleanlink, (int)reallen));
+			ret = False;
+		}
+	}
+
+	vfs_ChDir(conn, savedir);
+	
+	return ret;
+}
+
 /*******************************************************************
  Reduce a file name, removing .. elements and checking that
  it is below dir in the heirachy. This uses vfs_GetWd() and so must be run
@@ -743,7 +806,7 @@ BOOL reduce_name(connection_struct *conn, char *s,char *dir,BOOL widelinks)
 	p = strrchr(base_name,'/');
 
 	if (!p)
-		return(True);
+		return readlink_check(conn, dir, s);
 
 	if (!vfs_GetWd(conn,wd)) {
 		DEBUG(0,("couldn't vfs_GetWd for %s %s\n",s,dir));
@@ -777,7 +840,7 @@ BOOL reduce_name(connection_struct *conn, char *s,char *dir,BOOL widelinks)
 
 	if (!vfs_GetWd(conn,newname)) {
 		vfs_ChDir(conn,wd);
-		DEBUG(2,("couldn't get vfs_GetWd for %s %s\n",s,dir2));
+		DEBUG(2,("couldn't get vfs_GetWd for %s %s\n",s,base_name));
 		return(False);
 	}
 
@@ -797,6 +860,11 @@ BOOL reduce_name(connection_struct *conn, char *s,char *dir,BOOL widelinks)
 			return(False);
 		}
 
+		if (!readlink_check(conn, dir, newname)) {
+			DEBUG(2, ("Bad access attemt? %s is a symlink outside the share path", s));
+			return(False);
+		}
+
 		if (relative) {
 			if (newname[l] == '/')
 				pstrcpy(s,newname + l + 1);
@@ -815,3 +883,4 @@ BOOL reduce_name(connection_struct *conn, char *s,char *dir,BOOL widelinks)
 	return(True);
 #endif
 }
+
