@@ -165,6 +165,7 @@ static void print_brl(SMB_DEV_T dev, SMB_INO_T ino, int pid,
   ******************************************************************/
 static int profile_dump(void)
 {
+#ifdef WITH_PROFILE
 	if (!profile_setup(True)) {
 		fprintf(stderr,"Failed to initialise profile memory\n");
 		return -1;
@@ -193,6 +194,11 @@ static int profile_dump(void)
 	printf("write_count:                    %u\n", profile_p->syscall_write_count);
 	printf("write_time:                     %u\n", profile_p->syscall_write_time);
 	printf("write_bytes:                    %u\n", profile_p->syscall_write_bytes);
+#ifdef WITH_SENDFILE
+	printf("sendfile_count:                    %u\n", profile_p->syscall_sendfile_count);
+	printf("sendfile_time:                     %u\n", profile_p->syscall_sendfile_time);
+	printf("sendfile_bytes:                    %u\n", profile_p->syscall_sendfile_bytes);
+#endif
 	printf("lseek_count:                    %u\n", profile_p->syscall_lseek_count);
 	printf("lseek_time:                     %u\n", profile_p->syscall_lseek_time);
 	printf("rename_count:                   %u\n", profile_p->syscall_rename_count);
@@ -502,6 +508,9 @@ static int profile_dump(void)
 	printf("election_count:                 %u\n", profile_p->election_count);
 	printf("election_time:                  %u\n", profile_p->election_time);
 
+#else /* ndef WITH_PROFILE */
+	fprintf(stderr,"Profile data unavailable\n");
+#endif /* WITH_PROFILE */
 	return 0;
 }
 
@@ -641,12 +650,15 @@ static int traverse_fn1(TDB_CONTEXT *tdb, TDB_DATA kbuf, TDB_DATA dbuf, void *st
 	
 	tdb = tdb_open_log(lock_path("connections.tdb"), 0, TDB_DEFAULT, O_RDONLY, 0);
 	if (!tdb) {
-		printf("connections.tdb not initialised\n");
-		if (!lp_status(-1))
+		printf("%s not initialized.\n", lock_path("connections.tdb"));
+		printf("This is normal if an SMB client has never connected to your server.\n");
+		if (!lp_status(-1)) {
 			printf("You need to have status=yes in your smb config file\n");
+		}
 		return(0);
 	}  else if (verbose) {
-		printf("Opened status file %s\n", fname);
+		slprintf (fname, sizeof(fname)-1, "%s/%s", lp_lockdir(), "connections.tdb");
+		printf("Opened %s\n", fname);
 	}
 
 	if (locks_only) goto locks;
@@ -660,6 +672,7 @@ static int traverse_fn1(TDB_CONTEXT *tdb, TDB_DATA kbuf, TDB_DATA dbuf, void *st
 		printf("----------------------------------------------\n");
 	}
 	tdb_traverse(tdb, traverse_fn1, NULL);
+	
 
  locks:
 	if (processes_only) exit(0);
